(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "request", "zlib"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var request = require("request");
    var zlib = require("zlib");
    var HttpClient = (function () {
        function HttpClient(cfg, logger) {
            this.cfg = cfg;
            this.logger = logger;
            if (!cfg) {
                throw new Error("cfg is required");
            }
            if (!logger) {
                throw new Error("logger is required");
            }
        }
        HttpClient.prototype.get = function (urlPath, callback) {
            var opts = this.createDefaultOptions(urlPath);
            opts.json = true;
            request.get(opts, function (err, response, data) {
                if (err) {
                    return callback(err, null);
                }
                if (response.statusCode == 404) {
                    return callback(null, null);
                }
                if (response.statusCode != 200) {
                    return callback(new Error("Server returned HTTP Status " + response.statusCode), null);
                }
                return callback(null, data);
            });
        };
        HttpClient.prototype.delete = function (urlPath, callback) {
            var opts = this.createDefaultOptions(urlPath);
            opts.json = true;
            request.delete(opts, function (err, response, data) {
                if (err) {
                    return callback(err, null);
                }
                if (response.statusCode == 404) {
                    return callback(null, null);
                }
                if (response.statusCode != 200) {
                    return callback(new Error("Server returned HTTP Status " + response.statusCode), null);
                }
                return callback(null, data);
            });
        };
        HttpClient.prototype.post = function (bufferToSend, urlPath, callback) {
            var _this = this;
            var shouldZip = this.cfg.compressRequests;
            if (shouldZip) {
                zlib.gzip(bufferToSend, function (err, compressedBuf) {
                    if (err) {
                        _this.logger.warn("Failed while trying to compress the request data. Sending uncompressed data instead. Error: ", err);
                        _this.internalPost(urlPath, bufferToSend, null, callback);
                    }
                    else {
                        _this.internalPost(urlPath, compressedBuf, { "Content-Encoding": "gzip" }, callback);
                    }
                });
            }
            else {
                this.internalPost(urlPath, bufferToSend, null, callback);
            }
        };
        HttpClient.prototype.internalPost = function (urlPath, buffer, additionalHeaders, callback) {
            try {
                var opts = this.createDefaultOptions(urlPath);
                opts.body = buffer;
                if (additionalHeaders) {
                    for (var header in additionalHeaders) {
                        opts.headers[header] = additionalHeaders[header];
                    }
                }
                request.post(opts, function (err, response, data) {
                    if (err) {
                        return callback(err);
                    }
                    if (response.statusCode >= 400) {
                        return callback(new Error("HTTP error " + response.statusCode));
                    }
                    return callback(null);
                });
            }
            catch (err) {
                this.logger.error("Failed sending Http POST to :'" + urlPath + "'. Error:" + err);
                callback(err);
            }
        };
        HttpClient.prototype.allowUntrustedCertificates = function () {
            process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
        };
        HttpClient.prototype.createDefaultOptions = function (urlPath) {
            var opts = {
                url: this.cfg.server + urlPath,
                headers: {
                    "Content-Type": "application/json",
                    "Authorization": "Bearer " + this.cfg.token
                }
            };
            opts["compressed"] = true;
            if (this.cfg.proxy != null) {
                opts.proxy = this.cfg.proxy;
                this.allowUntrustedCertificates();
            }
            return opts;
        };
        return HttpClient;
    }());
    exports.HttpClient = HttpClient;
});
//# sourceMappingURL=http-client.js.map